/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */
package com.ibm.ts.alerts.delivery;

import java.io.UnsupportedEncodingException;
import java.util.Date;

import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.naming.NamingException;

import com.ibm.ts.alerts.delivery.EmailDeliveryMethod;

/**
 * Encapsulates an e-mail message; used by {@link EmailDeliveryMethod}.  Instances of this
 * class implement the {@link #send} method, which will send an e-mail to the specified
 * addressee.
 */
public class Email
{
  private String              to;
  private String              subject;
  private String              from;
  private String              fromPersonal;
  private String              cc;
  private String              bcc;
  private String              priority;
  private String              recordFolder;
  private String              text;

  public Email(String to, String subject, String from, String fromPersonal, String cc, String bcc, String priority, String recordFolder, String text)
  {
    this.to = to;
    this.subject = subject;
    this.from = from;
    this.fromPersonal = fromPersonal;
    this.cc = cc;
    this.bcc = bcc;
    this.priority = priority;
    this.recordFolder = recordFolder;
    this.text = text;
  }

  /**
   * Send this message to its recipient
   */
  public void send()
  {
    try
    {
      javax.naming.InitialContext ctx = new javax.naming.InitialContext();

      javax.mail.Session session = (javax.mail.Session) ctx.lookup("mail/AlertsMailSession"); //$NON-NLS-1$
      MimeMessage message = new MimeMessage(session);

      if ( from != null )
      {
        final InternetAddress fromAdress = new InternetAddress(from);
        if ( fromPersonal != null )
        {
          fromAdress.setPersonal(fromPersonal);
        }
        message.setFrom(fromAdress);
      }
      else
      {
        message.setFrom();
      }

      message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
      if ( cc != null )
      {
        message.setRecipients(Message.RecipientType.CC, InternetAddress.parse(cc, false));
      }
      if ( bcc != null )
      {
        message.setRecipients(Message.RecipientType.BCC, InternetAddress.parse(bcc, false));
      }

      if ( priority != null )
      {
        /*
         * Example value: "1" for high priority
         */
        message.addHeader("X-Priority", priority); //$NON-NLS-1$
      }

      message.setSubject(subject);
      message.setText(text);
      message.setSentDate(new Date());

      Transport.send(message);

      // Keep a copy, if requested.
      if ( recordFolder != null )
      {
        // Get a Store object
        Store store = session.getStore();

        store.connect();

        // Get record Folder. Create if it does not exist.
        final Folder folder = store.getFolder(recordFolder);
        if ( folder == null )
        {
          System.err.println("Null folder object. Can't record the email in the folder " + recordFolder); //$NON-NLS-1$
        }

        if ( !folder.exists() )
        {
          folder.create(Folder.HOLDS_MESSAGES);
        }

        final Message[] messages = new Message[1];
        messages[0] = message;
        folder.appendMessages(messages);
      }
    }
    catch ( MessagingException e )
    {
      System.err.println(e);
    }
    catch ( NamingException e )
    {
      System.err.println(e);
    }
    catch ( UnsupportedEncodingException e )
    {
      System.err.println(e);
    }
  }

  /**
   * @param bcc
   *          The bcc to set.
   */
  public void setBcc(String bcc)
  {
    this.bcc = bcc;
  }

  /**
   * @param cc
   *          The cc to set.
   */
  public void setCc(String cc)
  {
    this.cc = cc;
  }

  /**
   * @param from
   *          The from to set.
   */
  public void setFrom(String from)
  {
    this.from = from;
  }

  /**
   * @param fromPersonal
   *          The fromPersonal to set.
   */
  public void setFromPersonal(String fromPersonal)
  {
    this.fromPersonal = fromPersonal;
  }

  /**
   * @param record
   *          The record to set.
   */
  public void setRecordFolder(String recordFolder)
  {
    this.recordFolder = recordFolder;
  }

  /**
   * @param subject
   *          The subject to set.
   */
  public void setSubject(String subject)
  {
    this.subject = subject;
  }

  /**
   * @param text
   *          The text to set.
   */
  public void setText(String text)
  {
    this.text = text;
  }

  /**
   * @param to
   *          The to to set.
   */
  public void setTo(String to)
  {
    this.to = to;
  }

  /**
   * @return Returns the priority.
   */
  public String getPriority()
  {
    return priority;
  }

  /**
   * @param priority
   *          The priority to set.
   */
  public void setPriority(String priority)
  {
    this.priority = priority;
  }
}
