/*
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */

package com.ibm.ts.alerts.nativealerts;

import static com.ibm.rfidic.common.epcis.EventConstants.*;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.IParameterValue;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.path.PathDictionary;
import com.ibm.rfidic.value.IAttributeType;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;
import com.ibm.rfidic.value.ValueFactory;

/**
 * This class is a sample Native Alert provided to show how a Native Alert
 * should be built. It sends a notification when totes of C2 drugs enters 
 * the pharmacy.
 * <p>There are 2 input parameters:
 * <ol>
 *   <li>Read point ID of front door<li>
 *   </li>EPC class of totes of C2 drugs entering the pharmacy</li>
 * </ol>
 * <p>It has 5 output parameters:
 * <ol>
 *  <li>Event id</li>
 *  <li>Event type</li>
 *  <li>Event time</li>
 *  <li>Read point</li>
 * <li>Event constants</li>
 * </ol>
 */
public class CaseEnteredFrontDoorAlert extends SuperNativeAlert {

  private static final String BIZ_STEP_PARAM_NAME = "bizStep";

  private static final String EPC_PREFIX_PARAM_NAME = "epcPrefix";

  /* Declare arrays for the input and output parameters. */
  private IParameter[] inputParameters;

  private IParameter[] outputParameters;

    public CaseEnteredFrontDoorAlert() {

        /*
         * Call the super class constructor passing in the name and description
         * of the alert. This text will appear in the Alerts Subscription wizard
         * pages.
         */
        super(
                "Case Entered Front Door alert", //$NON-NLS-1$
                "Detects the event when totes/cases of drugs enter through the front " //$NON-NLS-1$
                        + "door of the pharmacy, instead of the shipping dock."); //$NON-NLS-1$

        /*
         * Declare an input parameter array for our alert definition. These
         * parameters will be provided by the subscription and entered by the
         * user into the third page of the Alerts Subscription wizard.
         */
    /*
     * Input parameters:
     * 
     * 1. Read point ID of front door
     * 
     * 2. EPC class of totes (of C2 drugs) entering the pharmacy
     * 
     * 3. Receiving business step name
     */
    final IAttributeType stringType = TypeFactory.getInstance ().getStringType (255, 0);
    this.inputParameters = new IParameter[3];
        this.inputParameters[0] = new Parameter(readPoint,
                stringType, true, null);
        this.inputParameters[1] = new Parameter(EPC_PREFIX_PARAM_NAME,
                stringType, true, null);
        this.inputParameters[2] = new Parameter (
                                                 BIZ_STEP_PARAM_NAME,       // Name
                                                 stringType,      // Type - string
                                                 false,           // Required - no
                                                 ValueFactory.getValue("urn:epcglobal:cbv:bizstep:receiving"));

        /*
         * Declare an output parameter array for the Alert. These parameters
         * will be placed in the returned Notification and optionally used by
         * the Delivery Method.
         */
        /*
         * Output parameters:
         * 
         * 1. Event time (need to figure out what time we exactly want to show
         * here)
         * 
         * 2. Read point ID of the reader where the drug with this EPC was last
         * seen
         * 
         * 3. EPC of the drug that entered through the front door of the
         * pharmacy
     */
    this.outputParameters = new IParameter[3];
        this.outputParameters[0] = new Parameter(eventTime,
                                                 TypeFactory.getInstance().getDateType(), true, null);
        this.outputParameters[1] = new Parameter(readPoint,
                                                 stringType, true, null);
        this.outputParameters[2] = new Parameter(epc,
                                                 stringType, true, null);
    }

    /*
     * This method contains the condition to be evaluated for this Alert. It
     * returns a Notification.
     */
    protected List<INotification> evaluateCondition(IEvent event, ISubscription subscription) {

        List<INotification> notifications = new ArrayList<INotification>();

        System.out.println("CaseEnteredFrontDoorAlert event = " + event); //$NON-NLS-1$

        /* Retrieve the standard EPCIS bizstep attribute from the Event. */
        IAttributeValue bizStepValue = getAttributeValue(event,
                bizStep);
        String bizStep = (String) bizStepValue.getSQLValue();
        System.out.println("bizStep = " + bizStep); //$NON-NLS-1$

        /* if this bizStep is a receiving bizStep */
        final String recvBizStep = getParameterValue(subscription, BIZ_STEP_PARAM_NAME).toString ();
        if (bizStep.equals(recvBizStep)) {

            IAttributeValue eventTimeValue = null;
            IAttributeValue readPointValue = null;
            IAttributeValue epcValue = null;

            /* Retrieve the standard EPCIS eventTime attribute from the Event. */
            eventTimeValue = getAttributeValue(event, eventTime);
            System.out.println("eventTimeValue = " + eventTimeValue); //$NON-NLS-1$

            java.sql.Timestamp eventTime = (java.sql.Timestamp) eventTimeValue
                    .getSQLValue();
            System.out.println("eventTime = " + eventTime); //$NON-NLS-1$

            /* Retrieve the standard EPCIS readPoint attribute from the Event */
            readPointValue = getAttributeValue(event, readPoint);
            System.out.println("readPointValue = '" + readPointValue+"'"); //$NON-NLS-1$

            /* Retrieve the standard EPCIS epc attribute from the Event */
            epcValue = getAttributeValue(event, EventConstants.epc, "#0");
            System.out.println("epcValue = " + epcValue.toString ()); //$NON-NLS-1$

            final String caseEpcPrefix = getParameterValue (subscription, EPC_PREFIX_PARAM_NAME).toString ();
            final String frontDoorRP = getParameterValue (subscription, readPoint).toString ();
            if (frontDoorRP.equals (readPointValue.toString ()) && epcValue.toString ().startsWith (caseEpcPrefix)) {
              /*
               * Create a parameter values array for use in the output
               * notification.
               */
              ParameterValue[] paramValues = new ParameterValue[3];
              /* Populate the array with parameters and their values */
              paramValues[0] = new ParameterValue (outputParameters[0], eventTimeValue);
              paramValues[1] = new ParameterValue (outputParameters[1], readPointValue);
              paramValues[2] = new ParameterValue (outputParameters[2], epcValue);
              notifications.add (new Notification (subscription, paramValues));
            }
            System.out.println("CaseEnteredFrontDoor notifications = " + notifications.toString()); //$NON-NLS-1$
        }
        return notifications;
    }

    /*
     * This method just returns the input parameters. There is no need to alter
     * this method for Native Alerts.
     */
    public IParameter[] getInputParameters() {
    return inputParameters;
  }

  /*
     * This method just returns the output parameters. There is no need to alter
     * this method for Native Alerts.
     */
    public IParameter[] getOutputParameters() {
    return outputParameters;
  }
}
