/**
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-O98
 * (c)  Copyright IBM Corp. 2006, 2011 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * The following sample of source code ("Sample Code") is owned by 
 * International Business Machines Corporation or one of its subsidiaries 
 * ("IBM") and is copyrighted and licensed, not sold. You may use, copy, 
 * modify, and distribute the Sample Code in any form without payment to IBM, 
 * for the purpose of assisting you in the development of your applications.
 *
 * The Sample Code is provided to you on an "AS IS" basis, without warranty of 
 * any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
 * not allow for the exclusion or limitation of implied warranties, so the 
 * above limitations or exclusions may not apply to you. IBM shall not be 
 * liable for any damages you suffer as a result of using, copying, modifying 
 * or distributing the Sample Code, even if IBM has been advised of the 
 * possibility of such damages.
 */

package com.ibm.ts.alerts.nativealerts;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rfidic.alerts.INotification;
import com.ibm.rfidic.alerts.IParameter;
import com.ibm.rfidic.alerts.ISubscription;
import com.ibm.rfidic.alerts.Notification;
import com.ibm.rfidic.alerts.Parameter;
import com.ibm.rfidic.alerts.ParameterValue;
import com.ibm.rfidic.common.epcis.EventConstants;
import com.ibm.rfidic.event.IEvent;
import com.ibm.rfidic.value.IAttributeValue;
import com.ibm.rfidic.value.TypeFactory;

/**
 * This class is a sample Native Alert provided to show how a Native Alert
 * should be built. It sends a notification when drugs are stored outside of 
 * the vault. There are 5 input parameters - 1. EPC class of the C2 drugs (if 
 * there is more than one EPC class, create separate alerts), 2. Time interval 
 * that C2 drugs can be outside of the vault, 3. BizLocation ID of the vault, 
 * 4. Read point ID of the reader where the bottles are unpacked from the totes 
 * (i.e. the receiving place), and 5. Read point ID of the dispenser. There are 
 * 4 output parameters - 1. Event type, 2. Event time (need to figure out what time 
 * we exactly want to show here), 3. EPC of the drug that is in an unsafe place, and 
 * 4. Read point ID of the reader where the drug with this EPC was last seen.  
 */

public class DrugNotInSafeAreaAlert extends SuperNativeAlert {

    /* Declare arrays for the input and output parameters. */
  private IParameter[] inputParameters;

  private IParameter[] outputParameters;

    public DrugNotInSafeAreaAlert() {
        /*
         * Call the super class constructor passing in the name and description
         * of the alert. This text will appear in the Alerts Subscription wizard
         * pages.
         */
        super(
                "Drug Not In Safe Area", //$NON-NLS-1$
    "Sample alert to notify when C2 drugs are in an unsafe area in the pharmacy for an exceedingly long time"); //$NON-NLS-1$

    /*
         * Declare an input parameter array for our alert definition. These
         * parameters will be provided by the subscription and entered by the
         * user into the third page of the Alerts Subscription wizard.
         */
        /*
     * Input parameters:
     * 
         * 1. EPC class of the C2 drugs (if there is more than one EPC class,
         * create separate alerts)
     * 
     * 2. Time interval that C2 drugs can be outside of the vault
     * 
     * 3. BizLocation ID of the vault
     * 
         * 4. Read point ID of the reader where the bottles are unpacked from
         * the totes (the receiving place).
     * 
     * 5. Read point ID of the dispenser
     */
    this.inputParameters = new IParameter[5];
        this.inputParameters[0] = new Parameter("epcPrefix", TypeFactory //$NON-NLS-1$
                .getInstance().getStringType(255, 0), true, null);
        this.inputParameters[1] = new Parameter("timeOutsideVault", TypeFactory //$NON-NLS-1$
                .getInstance().getIntegerType(), true, null);
        this.inputParameters[2] = new Parameter("bizLocationVault", TypeFactory //$NON-NLS-1$
                .getInstance().getStringType(255, 0), true, null);
        this.inputParameters[3] = new Parameter("readPointReceivingArea", //$NON-NLS-1$
                TypeFactory.getInstance().getStringType(255, 0), true, null);
        this.inputParameters[4] = new Parameter("readPointDispenser", //$NON-NLS-1$
                TypeFactory.getInstance().getStringType(255, 0), true, null);

        /*
         * Declare an output parameter array for the Alert. These parameters
         * will be placed in the returned Notification and optionally used by
         * the Delivery Method.
         */
    /*
     * Output parameters:
     * 
     * 1. Event type
     * 
         * 2. Event time (need to figure out what time we exaclty want to show
         * here)
     * 
     * 3. EPC of the drug that is in an unsafe place
     * 
         * 4. Read point ID of the reader where the drug with this EPC was last
         * seen
         */
        this.outputParameters = new IParameter[4];
        this.outputParameters[0] = new Parameter(EventConstants.eventType,
                TypeFactory.getInstance().getIntegerType(), true, null);
        this.outputParameters[1] = new Parameter(EventConstants.eventTime,
                TypeFactory.getInstance().getStringType(255, 0), true, null);
        this.outputParameters[2] = new Parameter(EventConstants.epc,
                TypeFactory.getInstance().getDateType(), true, null);
        this.outputParameters[3] = new Parameter(EventConstants.readPoint,
                TypeFactory.getInstance().getStringType(255, 0), true, null);
    }

    /*
     * This method contains the condition to be evaluated for this Alert. It
     * returns a Notification.
     */
    protected List<INotification> evaluateCondition(IEvent event, ISubscription subscription) {

        List<INotification> notifications = new ArrayList<INotification>();

        System.out.println("event = " + event); //$NON-NLS-1$

        IAttributeValue eventTypeValue = null;
        IAttributeValue eventTimeValue = null;
        IAttributeValue epcValue = null;
        IAttributeValue readPointValue = null;

        /* Retrieve the standard EPCIS eventType attribute from the Event */
        eventTypeValue = getAttributeValue(event, EventConstants.eventType);
        System.out.println("eventTypeValue = " + eventTypeValue); //$NON-NLS-1$

        /* Retrieve the standard EPCIS eventTime attribute from the Event. */
        eventTimeValue = getAttributeValue(event, EventConstants.eventTime);
        System.out.println("eventTimeValue = " + eventTimeValue); //$NON-NLS-1$

        java.sql.Timestamp eventTime = (java.sql.Timestamp) eventTimeValue
                .getSQLValue();
        System.out.println("eventTime = " + eventTime); //$NON-NLS-1$

        /* Retrieve the standard EPCIS epc attribute from the Event */
        epcValue = getAttributeValue(event, EventConstants.epc);
        System.out.println("epcValue = " + epcValue); //$NON-NLS-1$

        /* Retrieve the standard EPCIS readPoint attribute from the Event */
        readPointValue = getAttributeValue(event, EventConstants.readPoint);
        System.out.println("readPointValue = " + readPointValue); //$NON-NLS-1$

        /*
         * Create a parameter values array for use in the output notification.
         */
        ParameterValue[] paramValues = new ParameterValue[4];

        /* Populate the array with parameters and their values */
        paramValues[1] = new ParameterValue(outputParameters[1], eventTypeValue);
        paramValues[2] = new ParameterValue(outputParameters[2], eventTimeValue);
        paramValues[3] = new ParameterValue(outputParameters[3], readPointValue);
        paramValues[4] = new ParameterValue(outputParameters[4], epcValue);

        notifications.add(new Notification(subscription, paramValues));

        System.out.println("notifications = " + notifications.toString()); //$NON-NLS-1$

        return notifications;
    }

    /*
     * This method just returns the input parameters. There is no need to alter
     * this method for Native Alerts.
     */
    public IParameter[] getInputParameters() {
    return inputParameters;
  }

  /*
     * This method just returns the output parameters. There is no need to alter
     * this method for Native Alerts.
     */
    public IParameter[] getOutputParameters() {
    return outputParameters;
  }
}
